'use strict';

var _classCallCheck = require('@babel/runtime/helpers/classCallCheck');
var _assertThisInitialized = require('@babel/runtime/helpers/assertThisInitialized');
var _inherits = require('@babel/runtime/helpers/inherits');
var _possibleConstructorReturn = require('@babel/runtime/helpers/possibleConstructorReturn');
var _getPrototypeOf = require('@babel/runtime/helpers/getPrototypeOf');
var utils = require('./utils.js');
var widget = require('./widget.js');
var valueChangeEvent = require('./events/value-change-event.js');
var textChangeEvent = require('./events/text-change-event.js');
var focusEvent = require('./events/focus-event.js');
var blurEvent = require('./events/blur-event.js');
var keyEvent = require('./events/key-event.js');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var _classCallCheck__default = /*#__PURE__*/_interopDefaultLegacy(_classCallCheck);
var _assertThisInitialized__default = /*#__PURE__*/_interopDefaultLegacy(_assertThisInitialized);
var _inherits__default = /*#__PURE__*/_interopDefaultLegacy(_inherits);
var _possibleConstructorReturn__default = /*#__PURE__*/_interopDefaultLegacy(_possibleConstructorReturn);
var _getPrototypeOf__default = /*#__PURE__*/_interopDefaultLegacy(_getPrototypeOf);

function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf__default['default'](Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf__default['default'](this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn__default['default'](this, result); }; }

function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

var BoxField = /*#__PURE__*/function (_Widget) {
  _inherits__default['default'](BoxField, _Widget);

  var _super = _createSuper(BoxField);

  function BoxField(box, shell) {
    var _this;

    _classCallCheck__default['default'](this, BoxField);

    if (!box) {
      box = document.createElement('input');
      box.type = 'text';
    }

    if (!shell) {
      shell = box;
    }

    _this = _super.call(this, box, shell);
    _this.focusable = true;

    var self = _assertThisInitialized__default['default'](_this);
    /**
     * The text to be shown when component's value is absent.
     */


    Object.defineProperty(_assertThisInitialized__default['default'](_this), 'emptyText', {
      configurable: true,
      get: function get() {
        return box.placeholder;
      },
      set: function set(aValue) {
        box.placeholder = aValue;
      }
    });

    function checkValidity() {
      return box.checkValidity();
    }

    _this.checkValidity = checkValidity;

    function formatError() {
      return box.validationMessage;
    }

    _this.formatError = formatError;

    function validate() {
      if (self.checkValidity) {
        if (self.checkValidity()) {
          if (self.hideError) {
            self.hideError();
          }
        } else {
          var message = self.error ? self.error : self.formatError ? self.formatError() : null;

          if (message && self.showError) {
            self.showError(message);
          }
        }
      }
    }

    _this.validate = validate;
    _this.validateOnInput = true;
    var changeReg = utils.on(box, utils.Events.CHANGE, function (evt) {
      if (self.validate) {
        self.validate();
      }

      if (self.textChanged) {
        self.textChanged();
      }
    });
    var enterReg = utils.on(box, utils.Events.KEYPRESS, function (evt) {
      if (evt.key.toLowerCase() === 'enter') {
        self.fireAction();
      }
    });
    var inputReg = utils.on(box, utils.Events.INPUT, function (evt) {
      if (self.validateOnInput) {
        if (self.validate) {
          self.validate();
        }
      }

      fireTextChanged(box.value);
    });

    function hideError() {
      if (errorPopup && errorPopup.parentNode) errorPopup.parentNode.removeChild(errorPopup);
      errorPopup = null;
    }

    _this.hideError = hideError;
    var errorPopup = null;

    function showError(errorText) {
      if (errorPopup == null) {
        errorPopup = document.createElement('div');
        errorPopup.className = 'p-error-popup';
        errorPopup.innerText = errorText;

        if (shell === box) {
          var left = utils.absoluteLeft(box);
          var top = utils.absoluteTop(box);
          errorPopup.style.left = "".concat(left, "px");
          errorPopup.style.top = "".concat(top, "px");
          document.body.appendChild(errorPopup);
          errorPopup.style.top = "".concat(top + box.offsetHeight, "px");
        } else {
          shell.appendChild(errorPopup);
        }

        utils.later(function () {
          errorPopup.className += ' p-error-popup-shown';
        });
      } else {
        errorPopup.innerText = errorText;
      }

      return errorPopup;
    }

    _this.showError = showError;
    Object.defineProperty(_assertThisInitialized__default['default'](_this), 'valid', {
      get: function get() {
        return self.checkValidity ? self.checkValidity() : box.checkValidity;
      }
    });
    var textChangeHandlers = new Set();

    function addTextChangeHandler(handler) {
      textChangeHandlers.add(handler);
      return {
        removeHandler: function removeHandler() {
          textChangeHandlers["delete"](handler);
        }
      };
    }

    Object.defineProperty(_assertThisInitialized__default['default'](_this), 'addTextChangeHandler', {
      get: function get() {
        return addTextChangeHandler;
      }
    });

    function fireTextChanged(aValue) {
      var event = new textChangeEvent(self, aValue);
      textChangeHandlers.forEach(function (h) {
        utils.later(function () {
          h(event);
        });
      });
    }

    Object.defineProperty(_assertThisInitialized__default['default'](_this), 'fireTextChanged', {
      get: function get() {
        return fireTextChanged;
      }
    });
    var onTextChange;
    var onTextChangeReg;
    Object.defineProperty(_assertThisInitialized__default['default'](_this), 'onTextChange', {
      get: function get() {
        return onTextChange;
      },
      set: function set(aValue) {
        if (onTextChange !== aValue) {
          if (onTextChangeReg) {
            onTextChangeReg.removeHandler();
            onTextChangeReg = null;
          }

          onTextChange = aValue;

          if (onTextChange && addTextChangeHandler) {
            onTextChangeReg = addTextChangeHandler(function (event) {
              if (onTextChange) {
                onTextChange(event);
              }
            });
          }
        }
      }
    });
    var valueChangeHandlers = new Set();

    function addValueChangeHandler(handler) {
      valueChangeHandlers.add(handler);
      return {
        removeHandler: function removeHandler() {
          valueChangeHandlers["delete"](handler);
        }
      };
    }

    Object.defineProperty(_assertThisInitialized__default['default'](_this), 'addValueChangeHandler', {
      get: function get() {
        return addValueChangeHandler;
      }
    });

    function fireValueChanged(oldValue) {
      var event = new valueChangeEvent(self, oldValue, self.value);
      valueChangeHandlers.forEach(function (h) {
        utils.later(function () {
          h(event);
        });
      });
    }

    Object.defineProperty(_assertThisInitialized__default['default'](_this), 'fireValueChanged', {
      get: function get() {
        return fireValueChanged;
      }
    });
    var focusHandlers = new Set();

    function addFocusHandler(handler) {
      focusHandlers.add(handler);
      return {
        removeHandler: function removeHandler() {
          focusHandlers["delete"](handler);
        }
      };
    }

    Object.defineProperty(_assertThisInitialized__default['default'](_this), 'addFocusHandler', {
      get: function get() {
        return addFocusHandler;
      }
    });
    utils.on(box, utils.Events.FOCUS, fireFocus);

    function fireFocus() {
      var event = new focusEvent(self);
      focusHandlers.forEach(function (h) {
        utils.later(function () {
          h(event);
        });
      });
    }

    var focusLostHandlers = new Set();

    function addFocusLostHandler(handler) {
      focusLostHandlers.add(handler);
      return {
        removeHandler: function removeHandler() {
          focusLostHandlers["delete"](handler);
        }
      };
    }

    Object.defineProperty(_assertThisInitialized__default['default'](_this), 'addFocusLostHandler', {
      get: function get() {
        return addFocusLostHandler;
      }
    });
    utils.on(box, utils.Events.BLUR, fireBlur);

    function fireBlur() {
      var event = new blurEvent(self);
      focusLostHandlers.forEach(function (h) {
        utils.later(function () {
          h(event);
        });
      });
    }

    function addKeyTypeHandler(handler) {
      return utils.on(box, utils.Events.KEYPRESS, function (evt) {
        handler(new keyEvent(self, evt));
      });
    }

    Object.defineProperty(_assertThisInitialized__default['default'](_this), 'addKeyTypeHandler', {
      configurable: true,
      get: function get() {
        return addKeyTypeHandler;
      }
    });

    function addKeyPressHandler(handler) {
      return utils.on(box, utils.Events.KEYDOWN, function (evt) {
        handler(new keyEvent(self, evt));
      });
    }

    Object.defineProperty(_assertThisInitialized__default['default'](_this), 'addKeyPressHandler', {
      configurable: true,
      get: function get() {
        return addKeyPressHandler;
      }
    });

    function addKeyReleaseHandler(handler) {
      return utils.on(box, utils.Events.KEYUP, function (evt) {
        handler(new keyEvent(self, evt));
      });
    }

    Object.defineProperty(_assertThisInitialized__default['default'](_this), 'addKeyReleaseHandler', {
      configurable: true,
      get: function get() {
        return addKeyReleaseHandler;
      }
    });
    return _this;
  }

  return BoxField;
}(widget);

module.exports = BoxField;
