'use strict';

var _classCallCheck = require('@babel/runtime/helpers/classCallCheck');
var utils = require('./utils.js');
var mouseEvent = require('./events/mouse-event.js');
var widgetEvent = require('./events/widget-event.js');
var actionEvent = require('./events/action-event.js');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var _classCallCheck__default = /*#__PURE__*/_interopDefaultLegacy(_classCallCheck);

var Widget = function Widget(box, shell) {
  _classCallCheck__default['default'](this, Widget);

  box = box || document.createElement('div');
  shell = shell || box;
  if (shell !== box) shell.appendChild(box);
  var self = this;
  shell['p-widget'] = this;
  shell.classList.add('p-widget');
  var parent;
  var visibleDisplay = '';
  var contextMenu;
  var enabled = true;
  var name;
  var background = null;
  var foreground = null;
  var cursor;
  var toolTipText;
  var focusable = false;
  var font = null;
  Object.defineProperty(this, 'visibleDisplay', {
    get: function get() {
      return visibleDisplay;
    },
    set: function set(aValue) {
      visibleDisplay = aValue;
    }
  });
  Object.defineProperty(this, 'parent', {
    get: function get() {
      return parent;
    },
    set: function set(aValue) {
      parent = aValue;
    }
  });
  Object.defineProperty(this, 'element', {
    get: function get() {
      return shell;
    }
  });
  Object.defineProperty(this, 'shell', {
    get: function get() {
      return shell;
    }
  });
  Object.defineProperty(this, 'box', {
    get: function get() {
      return box;
    }
  });
  Object.defineProperty(this, 'enabled', {
    get: function get() {
      return enabled;
    },
    set: function set(aValue) {
      if (enabled !== aValue) {
        enabled = aValue;
        if (enabled) box.removeAttribute('disabled');else box.setAttribute('disabled', '');
      }
    }
  });
  Object.defineProperty(this, 'name', {
    get: function get() {
      return name;
    },
    set: function set(aValue) {
      name = aValue;
      self.element.name = name;
    }
  });
  Object.defineProperty(this, 'title', {
    get: function get() {
      return box.title;
    },
    set: function set(aValue) {
      box.title = aValue;
    }
  });
  Object.defineProperty(this, 'font', {
    get: function get() {
      return font;
    },
    set: function set(aValue) {
      if (font !== aValue) {
        font = aValue;

        if (font) {
          shell.style.fontFamily = font.family;
          shell.style.fontSize = "".concat(font.size, "pt");

          if (font.bold) {
            shell.style.fontWeight = 'bold';
          } else {
            shell.style.fontWeight = 'normal';
          }

          if (font.italic) {
            shell.style.fontStyle = 'italic';
          } else {
            shell.style.fontStyle = 'normal';
          }
        } else {
          shell.style.fontFamily = '';
          shell.style.fontSize = '';
          shell.style.fontWeight = '';
          shell.style.fontStyle = '';
        }
      }
    }
  });
  Object.defineProperty(this, 'background', {
    get: function get() {
      return background;
    },
    set: function set(aValue) {
      if (background !== aValue) {
        background = aValue;
        box.style.backgroundColor = background && background.toStyled ? background.toStyled() : background ? background : '';
      }
    }
  });
  Object.defineProperty(this, 'foreground', {
    get: function get() {
      return foreground;
    },
    set: function set(aValue) {
      if (foreground !== aValue) {
        foreground = aValue;
        box.style.color = foreground && foreground.toStyled ? foreground.toStyled() : foreground ? foreground : '';
      }
    }
  });
  Object.defineProperty(this, 'cursor', {
    get: function get() {
      return cursor;
    },
    set: function set(aValue) {
      cursor = aValue;
      box.style.cursor = cursor;
    }
  });
  var errorText = null;
  Object.defineProperty(this, 'error', {
    get: function get() {
      return errorText;
    },
    set: function set(aValue) {
      if (errorText !== aValue) {
        errorText = aValue;
      }
    }
  });
  Object.defineProperty(this, 'toolTipText', {
    get: function get() {
      return toolTipText;
    },
    set: function set(aValue) {
      if (toolTipText !== aValue) {
        toolTipText = aValue;
        box.title = toolTipText;
      }
    }
  });
  var tabIndex = 1;

  function applyTabIndex() {
    if (focusable) {
      box.setAttribute('tabindex', "".concat(tabIndex));
    } else {
      box.removeAttribute('tabindex');
    }
  }

  Object.defineProperty(this, 'tabIndex', {
    get: function get() {
      return tabIndex;
    },
    set: function set(aValue) {
      if (!isNaN(aValue) && tabIndex !== aValue) {
        tabIndex = aValue;
        applyTabIndex();
      }
    }
  });
  Object.defineProperty(this, 'focusable', {
    get: function get() {
      return focusable;
    },
    set: function set(aValue) {
      if (focusable !== aValue) {
        focusable = aValue;
        applyTabIndex();
      }
    }
  });
  Object.defineProperty(this, 'left', {
    get: function get() {
      if (isAttached()) {
        return shell.offsetLeft;
      } else {
        var parsed = parseFloat(shell.style.left);
        return isNaN(parsed) ? 0 : parsed;
      }
    },
    set: function set(aValue) {
      if (aValue !== null) {
        if (parent && parent.ajustLeft) parent.ajustLeft(self, aValue);else shell.style.left = "".concat(aValue, "px");
      } else {
        shell.style.left = null;
      }
    }
  });
  Object.defineProperty(this, 'top', {
    get: function get() {
      if (isAttached()) {
        return shell.offsetTop;
      } else {
        var parsed = parseFloat(shell.style.top);
        return isNaN(parsed) ? 0 : parsed;
      }
    },
    set: function set(aValue) {
      if (aValue !== null) {
        if (parent && parent.ajustTop) parent.ajustTop(self, aValue);else shell.style.top = "".concat(aValue, "px");
      } else {
        shell.style.top = null;
      }
    }
  });
  Object.defineProperty(this, 'width', {
    get: function get() {
      if (isAttached()) return shell.offsetWidth;else {
        var parsed = parseFloat(shell.style.width);
        return isNaN(parsed) ? 0 : parsed;
      }
    },
    set: function set(aValue) {
      if (aValue !== null) {
        if (parent && parent.ajustWidth) {
          parent.ajustWidth(self, aValue);
        } else {
          shell.style.width = "".concat(aValue, "px");
        }
      } else {
        shell.style.width = null;
      }
    }
  });
  Object.defineProperty(this, 'height', {
    get: function get() {
      if (isAttached()) {
        return shell.offsetHeight;
      } else {
        var parsed = parseFloat(shell.style.height);
        return isNaN(parsed) ? 0 : parsed;
      }
    },
    set: function set(aValue) {
      if (aValue !== null) {
        if (parent && parent.ajustHeight) {
          parent.ajustHeight(self, aValue);
        } else {
          shell.style.height = "".concat(aValue, "px");
        }
      } else {
        shell.style.height = null;
      }
    }
  });
  var menuTriggerReg = null;
  Object.defineProperty(this, 'contextMenu', {
    get: function get() {
      return contextMenu;
    },
    set: function set(aValue) {
      if (contextMenu !== aValue) {
        if (menuTriggerReg) {
          menuTriggerReg.removeHandler();
          menuTriggerReg = null;
        }

        contextMenu = aValue;

        if (contextMenu) {
          menuTriggerReg = utils.on(box, utils.Events.CONTEXTMENU, function (event) {
            event.preventDefault();
            event.stopPropagation();

            if (contextMenu.showAt) {
              utils.startMenuSession(contextMenu);
              var pageX = 'pageX' in event ? event.pageX : event.clientX + document.body.scrollLeft + document.documentElement.scrollLeft;
              var pageY = 'pageY' in event ? event.pageY : event.clientY + document.body.scrollTop + document.documentElement.scrollTop;
              contextMenu.showAt(pageX, pageY);
            }
          });
        }
      }
    }
  });
  var actionHandlers = new Set();

  function addActionHandler(handler) {
    actionHandlers.add(handler);
    return {
      removeHandler: function removeHandler() {
        actionHandlers["delete"](handler);
      }
    };
  }

  function fireAction() {
    var event = new actionEvent(self);
    actionHandlers.forEach(function (h) {
      utils.later(function () {
        h(event);
      });
    });
  }

  var hideHandlers = new Set();

  function addHideHandler(handler) {
    hideHandlers.add(handler);
    return {
      removeHandler: function removeHandler() {
        hideHandlers["delete"](handler);
      }
    };
  }

  var showHandlers = new Set();

  function addShowHandler(handler) {
    showHandlers.add(handler);
    return {
      removeHandler: function removeHandler() {
        showHandlers["delete"](handler);
      }
    };
  }

  function addMouseClickHandler(handler) {
    return utils.on(shell, utils.Events.CLICK, function (evt) {
      handler(new mouseEvent(self, evt, 1));
    });
  }

  function addMouseDoubleClickHandler(handler) {
    return utils.on(shell, utils.Events.DBLCLICK, function (evt) {
      handler(new mouseEvent(self, evt, 2));
    });
  }

  function addMousePressHandler(h) {
    return utils.on(shell, utils.Events.MOUSEDOWN, function (evt) {
      h(new mouseEvent(self, evt));
    });
  }

  function addMouseReleaseHandler(handler) {
    return utils.on(shell, utils.Events.MOUSEUP, function (evt) {
      handler(new mouseEvent(self, evt));
    });
  }

  function addMouseMoveHandler(handler) {
    return utils.on(shell, utils.Events.MOUSEMOVE, function (evt) {
      handler(new mouseEvent(self, evt));
    });
  }

  function addMouseEnterHandler(handler) {
    return utils.on(shell, utils.Events.MOUSEOVER, function (evt) {
      handler(new mouseEvent(self, evt));
    });
  }

  function addMouseExitHandler(handler) {
    return utils.on(shell, utils.Events.MOUSEOUT, function (evt) {
      handler(new mouseEvent(self, evt));
    });
  }

  function addMouseWheelHandler(handler) {
    return utils.on(shell, utils.Events.MOUSEWHEEL, function (evt) {
      handler(new mouseEvent(self, evt));
    });
  }

  Object.defineProperty(this, 'addActionHandler', {
    configurable: true,
    get: function get() {
      return addActionHandler;
    }
  });
  Object.defineProperty(this, 'addHideHandler', {
    get: function get() {
      return addHideHandler;
    }
  });
  Object.defineProperty(this, 'addShowHandler', {
    get: function get() {
      return addShowHandler;
    }
  });
  Object.defineProperty(this, 'addMouseClickHandler', {
    get: function get() {
      return addMouseClickHandler;
    }
  });
  Object.defineProperty(this, 'addMouseDoubleClickHandler', {
    get: function get() {
      return addMouseDoubleClickHandler;
    }
  });
  Object.defineProperty(this, 'addMousePressHandler', {
    get: function get() {
      return addMousePressHandler;
    }
  });
  Object.defineProperty(this, 'addMouseReleaseHandler', {
    get: function get() {
      return addMouseReleaseHandler;
    }
  });
  Object.defineProperty(this, 'addMouseMoveHandler', {
    get: function get() {
      return addMouseMoveHandler;
    }
  });
  Object.defineProperty(this, 'addMouseEnterHandler', {
    get: function get() {
      return addMouseEnterHandler;
    }
  });
  Object.defineProperty(this, 'addMouseExitHandler', {
    get: function get() {
      return addMouseExitHandler;
    }
  });
  Object.defineProperty(this, 'addMouseWheelHandler', {
    get: function get() {
      return addMouseWheelHandler;
    }
  });
  Object.defineProperty(this, 'fireAction', {
    get: function get() {
      return fireAction;
    }
  });

  function isAttached() {
    var cursorElement = shell;

    while (cursorElement && cursorElement !== document.body) {
      cursorElement = cursorElement.parentElement;
    }

    return !!cursorElement;
  }

  Object.defineProperty(this, 'attached', {
    get: function get() {
      return isAttached();
    }
  });
  Object.defineProperty(this, 'visible', {
    get: function get() {
      return shell.style.display !== 'none';
    },
    set: function set(aValue) {
      var oldValue = self.visible;

      if (oldValue !== aValue) {
        if (aValue) {
          shell.style.display = visibleDisplay;
        } else {
          shell.style.display = 'none';
        }

        if (aValue) {
          fireShown();
        } else {
          fireHidden();
        }
      }
    }
  });

  function focus() {
    box.focus();
  }

  Object.defineProperty(this, 'focus', {
    get: function get() {
      return focus;
    }
  });

  function blur() {
    box.blur();
  }

  Object.defineProperty(this, 'blur', {
    get: function get() {
      return blur;
    }
  });

  function fireShown() {
    var event = new widgetEvent(self);
    showHandlers.forEach(function (h) {
      utils.later(function () {
        h(event);
      });
    });
  }

  function fireHidden() {
    var event = new widgetEvent(self);
    hideHandlers.forEach(function (h) {
      utils.later(function () {
        h(event);
      });
    });
  }

  var onAction;
  var actionReg;
  Object.defineProperty(this, 'onAction', {
    get: function get() {
      return onAction;
    },
    set: function set(aValue) {
      if (onAction !== aValue) {
        if (actionReg) {
          actionReg.removeHandler();
          actionReg = null;
        }

        onAction = aValue;

        if (onAction) {
          actionReg = self.addActionHandler(function (event) {
            if (onAction) {
              onAction(event);
            }
          });
        }
      }
    }
  });
  var onMouseExit;
  var mouseOutReg;
  Object.defineProperty(this, 'onMouseExit', {
    get: function get() {
      return onMouseExit;
    },
    set: function set(aValue) {
      if (onMouseExit !== aValue) {
        if (mouseOutReg) {
          mouseOutReg.removeHandler();
          mouseOutReg = null;
        }

        onMouseExit = aValue;

        if (onMouseExit) {
          mouseOutReg = addMouseExitHandler(function (evt) {
            if (onMouseExit) {
              evt.event.stopPropagation();
              onMouseExit(evt);
            }
          });
        }
      }
    }
  });
  var onMouseClick;
  var mouseClickReg;
  Object.defineProperty(this, 'onMouseClick', {
    get: function get() {
      return onMouseClick;
    },
    set: function set(aValue) {
      if (onMouseClick !== aValue) {
        if (mouseClickReg) {
          mouseClickReg.removeHandler();
          mouseClickReg = null;
        }

        onMouseClick = aValue;

        if (onMouseClick) {
          mouseClickReg = addMouseClickHandler(function (evt) {
            if (onMouseClick) {
              evt.event.stopPropagation();
              onMouseClick(evt);
            }
          });
        }
      }
    }
  });
  var onMouseDoubleClick;
  var mouseDoubleClickReg;
  Object.defineProperty(this, 'onMouseDoubleClick', {
    get: function get() {
      return onMouseDoubleClick;
    },
    set: function set(aValue) {
      if (onMouseDoubleClick !== aValue) {
        if (mouseDoubleClickReg) {
          mouseDoubleClickReg.removeHandler();
          mouseDoubleClickReg = null;
        }

        onMouseDoubleClick = aValue;

        if (onMouseDoubleClick) {
          mouseDoubleClickReg = addMouseDoubleClickHandler(function (evt) {
            if (onMouseDoubleClick) {
              evt.event.stopPropagation();
              onMouseDoubleClick(evt);
            }
          });
        }
      }
    }
  });
  var onMousePress;
  var mousePressReg;
  Object.defineProperty(this, 'onMousePress', {
    get: function get() {
      return onMousePress;
    },
    set: function set(aValue) {
      if (onMousePress !== aValue) {
        if (mousePressReg) {
          mousePressReg.removeHandler();
          mousePressReg = null;
        }

        onMousePress = aValue;

        if (onMousePress) {
          mousePressReg = addMousePressHandler(function (evt) {
            if (onMousePress) {
              evt.event.stopPropagation();
              onMousePress(evt);
            }
          });
        }
      }
    }
  });
  var onMouseRelease;
  var mouseReleaseReg;
  Object.defineProperty(this, 'onMouseRelease', {
    get: function get() {
      return onMouseRelease;
    },
    set: function set(aValue) {
      if (onMouseRelease !== aValue) {
        if (mouseReleaseReg) {
          mouseReleaseReg.removeHandler();
          mouseReleaseReg = null;
        }

        onMouseRelease = aValue;

        if (onMouseRelease) {
          mouseReleaseReg = addMouseReleaseHandler(function (evt) {
            if (onMouseRelease) {
              evt.event.stopPropagation();
              onMouseRelease(evt);
            }
          });
        }
      }
    }
  });
  var onMouseEnter;
  var mouseEnterReg;
  Object.defineProperty(this, 'onMouseEnter', {
    get: function get() {
      return onMouseEnter;
    },
    set: function set(aValue) {
      if (onMouseEnter !== aValue) {
        if (mouseEnterReg) {
          mouseEnterReg.removeHandler();
          mouseEnterReg = null;
        }

        onMouseEnter = aValue;

        if (onMouseEnter) {
          mouseEnterReg = addMouseEnterHandler(function (evt) {
            if (onMouseEnter) {
              evt.event.stopPropagation();
              onMouseEnter(evt);
            }
          });
        }
      }
    }
  });
  var onMouseWheelMove;
  var mouseWheelReg;
  Object.defineProperty(this, 'onMouseWheelMove', {
    get: function get() {
      return onMouseWheelMove;
    },
    set: function set(aValue) {
      if (onMouseWheelMove !== aValue) {
        if (mouseWheelReg) {
          mouseWheelReg.removeHandler();
          mouseWheelReg = null;
        }

        onMouseWheelMove = aValue;

        if (onMouseWheelMove) {
          mouseWheelReg = addMouseWheelHandler(function (evt) {
            if (onMouseWheelMove) {
              evt.event.stopPropagation();
              onMouseWheelMove(evt);
            }
          });
        }
      }
    }
  });
  var onMouseMove;
  var mouseMoveReg;
  Object.defineProperty(this, 'onMouseMove', {
    get: function get() {
      return onMouseMove;
    },
    set: function set(aValue) {
      if (onMouseMove !== aValue) {
        if (mouseMoveReg) {
          mouseMoveReg.removeHandler();
          mouseMoveReg = null;
        }

        onMouseMove = aValue;

        if (onMouseMove) {
          mouseMoveReg = addMouseMoveHandler(function (evt) {
            if (onMouseMove) {
              evt.event.stopPropagation();
              onMouseMove(evt);
            }
          });
        }
      }
    }
  });
  var onShow;
  var showReg;
  Object.defineProperty(this, 'onShow', {
    get: function get() {
      return onShow;
    },
    set: function set(aValue) {
      if (onShow !== aValue) {
        if (showReg) {
          showReg.removeHandler();
          showReg = null;
        }

        onShow = aValue;

        if (onShow) {
          showReg = addShowHandler(function (event) {
            if (onShow) {
              onShow(event);
            }
          });
        }
      }
    }
  });
  var onHide;
  var hideReg;
  Object.defineProperty(this, 'onHide', {
    get: function get() {
      return onHide;
    },
    set: function set(aValue) {
      if (onHide !== aValue) {
        if (hideReg) {
          hideReg.removeHandler();
          hideReg = null;
        }

        onHide = aValue;

        if (onHide) {
          hideReg = addHideHandler(function (event) {
            if (onHide) {
              onHide(event);
            }
          });
        }
      }
    }
  });
  var onFocus;
  var focusReg;
  Object.defineProperty(this, 'onFocus', {
    get: function get() {
      return onFocus;
    },
    set: function set(aValue) {
      if (onFocus !== aValue) {
        if (focusReg) {
          focusReg.removeHandler();
          focusReg = null;
        }

        onFocus = aValue;

        if (onFocus && self.addFocusHandler) {
          focusReg = self.addFocusHandler(function (event) {
            if (onFocus) {
              onFocus(event);
            }
          });
        }
      }
    }
  });
  var onFocusLost;
  var focusLostReg;
  Object.defineProperty(this, 'onFocusLost', {
    get: function get() {
      return onFocusLost;
    },
    set: function set(aValue) {
      if (onFocusLost !== aValue) {
        if (focusLostReg) {
          focusLostReg.removeHandler();
          focusLostReg = null;
        }

        onFocusLost = aValue;

        if (onFocusLost && self.addFocusLostHandler) {
          focusLostReg = self.addFocusLostHandler(function (event) {
            if (onFocusLost) {
              onFocusLost(event);
            }
          });
        }
      }
    }
  });
  var onKeyType;
  var keyTypeReg;
  Object.defineProperty(this, 'onKeyType', {
    get: function get() {
      return onKeyType;
    },
    set: function set(aValue) {
      if (onKeyType !== aValue) {
        if (keyTypeReg) {
          keyTypeReg.removeHandler();
          keyTypeReg = null;
        }

        onKeyType = aValue;

        if (onKeyType && self.addKeyTypeHandler) {
          keyTypeReg = self.addKeyTypeHandler(function (event) {
            if (onKeyType) {
              event.event.stopPropagation();
              onKeyType(event);
            }
          });
        }
      }
    }
  });
  var onKeyPress;
  var keyPressReg;
  Object.defineProperty(this, 'onKeyPress', {
    get: function get() {
      return onKeyPress;
    },
    set: function set(aValue) {
      if (onKeyPress !== aValue) {
        if (keyPressReg) {
          keyPressReg.removeHandler();
          keyPressReg = null;
        }

        onKeyPress = aValue;

        if (onKeyPress && self.addKeyPressHandler) {
          keyPressReg = self.addKeyPressHandler(function (event) {
            if (onKeyPress) {
              event.event.stopPropagation();
              onKeyPress(event);
            }
          });
        }
      }
    }
  });
  var onKeyRelease;
  var keyReleaseReg;
  Object.defineProperty(this, 'onKeyRelease', {
    get: function get() {
      return onKeyRelease;
    },
    set: function set(aValue) {
      if (onKeyRelease !== aValue) {
        if (keyReleaseReg) {
          keyReleaseReg.removeHandler();
          keyReleaseReg = null;
        }

        onKeyRelease = aValue;

        if (onKeyRelease && self.addKeyReleaseHandler) {
          keyReleaseReg = self.addKeyReleaseHandler(function (event) {
            if (onKeyRelease) {
              event.event.stopPropagation();
              onKeyRelease(event);
            }
          });
        }
      }
    }
  });
  var onSelect;
  var selectReg;
  Object.defineProperty(this, 'onSelect', {
    get: function get() {
      return onSelect;
    },
    set: function set(aValue) {
      if (onSelect !== aValue) {
        if (selectReg) {
          selectReg.removeHandler();
          selectReg = null;
        }

        onSelect = aValue;

        if (onSelect && self.addSelectHandler) {
          selectReg = self.addSelectHandler(function (event) {
            if (onSelect) {
              onSelect(event);
            }
          });
        }
      }
    }
  });
  var valueChange;
  var valueChangeReg;
  Object.defineProperty(this, 'onValueChange', {
    get: function get() {
      return valueChange;
    },
    set: function set(aValue) {
      if (valueChange !== aValue) {
        if (valueChangeReg) {
          valueChangeReg.removeHandler();
          valueChangeReg = null;
        }

        valueChange = aValue;

        if (valueChange && self.addValueChangeHandler) {
          valueChangeReg = self.addValueChangeHandler(function (event) {
            if (valueChange) {
              valueChange(event);
            }
          });
        }
      }
    }
  });
};

module.exports = Widget;
